/* capture.c (c) 2004 Petr Danecek
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#ifdef HAVE_READLINE
  #include <readline/readline.h>
  #include <readline/history.h>
#endif
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <fcntl.h>
#include <signal.h>
#include "capture.h"

int cmd_ok(char *cmd, char *ret)
{
      int n1, n2;
      char *arg1 = getarg(cmd, &n1);
      char *arg2 = getarg(ret, &n2);
      
      if ( n1 != n2 ) return 0;
      if ( strncasecmp(arg1, arg2, n1) ) return 0;

      arg2 = getarg(arg2+n2, &n2);
      if ( n2 != 2 ) return 0;
      if ( strncasecmp("ok", arg2, 2) ) return 0;

      return 1;
}


int run_command(char *str)
{
      char buf[MAX_CMD_LEN];
      if ( ! cmdcmp(str, CMD_HELP) ) 
      {
	    help(str);
	    return 0;
      }

      msg_send(str);

      while ( ! msg_get(buf, MAX_CMD_LEN) ) usleep(USLEEP);

      if ( ! cmd_ok(str, buf) ) 
      {
	    error("%s\n", buf);
	    return -1;
      }

      if ( ! cmdcmp(buf, CMD_QUIT) ) return 1;
      if ( ! cmdcmp(buf, CMD_LEAVE) ) return 1;
      
      if ( ! cmdcmp(buf, CMD_STATUS) ) printf("%s\n", buf);
      else if ( ! cmdcmp(buf, CMD_UNKNOWN) ) help(NULL);

      return 0;
}


void sigterm_handler(int signal, struct sigcontext sc)
{
      debug("client signal caught: %d\n", signal);
      run_command(CMD_QUIT);
      exit(1);
}

int run_client(int argc, char *argv[])
{
      int stat;
      char *s;
      char str[MAX_CMD_LEN];
      int i;

      debug("Client: Connected %d.\n", SOCKET);

      signal(SIGINT, (void (*)(int))sigterm_handler);

      for (i=0; i<argc; i++)
      {
	    stat = run_command(argv[i]);
	    if ( stat == -1 ) 
	    {
		  run_command(CMD_LEAVE);
		  return stat;
	    }
      }
      if ( i != 0 )
      {
	    if ( cmdcmp(argv[i-1], CMD_QUIT) )
		  run_command(CMD_LEAVE);

	    return 0;
      }


#ifdef HAVE_GTK
      if ( settings.viewfinder )
      {
	    run_command(CMD_START);
	    snprintf(str, MAX_CMD_LEN, "%s on", CMD_VF);
	    run_command(str);
      }
#endif


#ifdef HAVE_READLINE
      using_history();
#endif


      while( 1 ) 
      {
#ifdef HAVE_READLINE
	    s = readline("capture> ");
	    if ( ! s )
	    {
		  stat = run_command(CMD_LEAVE);
		  break;
	    }
	    if ( ! *s ) continue;
	    add_history(s);
	    stat = run_command(s);
#else
	    printf("capture> ");
	    fgets(str, MAX_CMD_LEN, stdin);
	    if ( feof(stdin) ) 
	    {
		    stat = run_command(CMD_LEAVE);
		    break;
	    }
	    if ( *str == '\n' ) continue;
	    
	    s = str;
	    while (str-s < MAX_CMD_LEN && *s) s++;
	    if ( s != str ) *(s-1) = 0;

	    stat = run_command(str);
#endif
	    if ( stat == 1 ) break;
      }

      close(SOCKET);

      if ( stat == -1 ) return stat;
      return 0;
}

